library geiger_localstorage;

import 'dart:async';

import 'package:geiger_localstorage/geiger_localstorage.dart';

class ExtendedTimestamp {
  /// offers the last know timestamp
  static int _lastTimestamp = 0;

  static int _lastWrite = 0;

  static bool _pendingWrite = true;

  static bool _init = false;

  static GenericController? _controller;

  static String? _uuid;

  static void reset() {
    _pendingWrite = false;
    _init = false;
    _uuid = null;
    _controller = null;
    _lastTimestamp = -1;
    _lastWrite = -1;
  }

  static int now([bool requireInit = true]) {
    if (!_init && requireInit) {
      throw StorageException('Extended timestamp not yet initialized');
    }
    int ret = DateTime.now().millisecondsSinceEpoch;
    if (ret <= _lastTimestamp) {
      ret = _lastTimestamp + 1;
    }
    _lastTimestamp = ret;
    _writeTimestampValueMicrotask();
    return ret;
  }

  static bool get init => _init;

  static Future<void> _writeTimestampValueMicrotask() async {
    if (_init &&
        _lastTimestamp + 10000 > _lastWrite &&
        _lastTimestamp > -1 &&
        !_pendingWrite) {
      _lastWrite = _lastTimestamp;
      _pendingWrite = true;
      scheduleMicrotask(() async {
        // make sure that the node exists and has not been zapped in the meantime
        Node n;
        try {
          n = await _controller!.mapper.get(':configTimestamp');
        } on StorageException {
          n = NodeImpl('configTimestamp', 'internal', '', Visibility.black);
          _lastTimestamp = now(false);
          await n.addValue(NodeValueImpl('lastUpdated', '$_lastTimestamp'));
          await _controller!.mapper.add(n);
        }
        // write updated value
        await n.updateValue(NodeValueImpl('lastUpdated', '$_lastTimestamp'));
        await _controller!.mapper.update(n);
        _pendingWrite = false;
      });
    }
  }

  /// Initializes the timestamp with the persisted and possibly un-persisted changes of the database.
  static Future<void> initializeTimestamp(StorageController controller) async {
    if (!_init) {
      _controller = controller as GenericController;
      Node n;
      try {
        n = await _controller!.mapper.get(':configTimestamp');
        if ((await n.getValue('lastUpdated')) == null) {
          _lastTimestamp = DateTime.now().millisecondsSinceEpoch;
          await n.addValue(NodeValueImpl('lastUpdated', '$_lastTimestamp'));
        }
      } on StorageException {
        n = NodeImpl('configTimestamp', 'internal', '', Visibility.black);
        _lastTimestamp = DateTime.now().millisecondsSinceEpoch;
        await n.addValue(NodeValueImpl('lastUpdated', '$_lastTimestamp'));
        try {
          await _controller!.mapper.add(n);
        } on StorageException {
          // someone else already added it... lets fetch it
          Node n2 = await _controller!.mapper.get(':configTimestamp');
          if((await n.getValue('lastUpdated'))!=null) {
            n=n2;
          }
        }
      }
      String? lastUpdated = (await n.getValue('lastUpdated'))!.toSimpleString();
      if (_uuid == null) {
        try {
          Node n2 = await _controller!.mapper.get(':Local');
          _uuid = (await n2.getValue('currentDevice'))!.toSimpleString();
        } on StorageException {
          // TODO(mgwerder): insert logger
        }
      }

      _lastTimestamp = int.parse(lastUpdated!);

      List<Node> nl = await (_controller as GenericController).mapper.search(
          SearchCriteria(searchPath: ':', nodeValueLastModified: lastUpdated));
      for (Node n in nl) {
        if (_lastTimestamp < n.lastModified) {
          _lastTimestamp = n.lastModified;
        }
      }
      _init = true;
    }
  }

  static String convertToExtendedTimestamp(int timestamp, [String? uuid]) {
    if (uuid != null) {
      _uuid = uuid;
    } else {
      uuid ??= _uuid;
    }
    return '${timestamp.toString().padLeft(20, '0')}-$_uuid';
  }
}
