// ignore_for_file: prefer_const_constructors

import 'package:geiger_localstorage/src/event_type.dart';
import 'package:geiger_localstorage/src/search_criteria.dart';
import 'package:geiger_localstorage/src/storage_controller.dart';
import 'package:geiger_localstorage/src/storage_exception.dart';
import 'package:geiger_localstorage/src/storage_listener.dart';
import 'package:geiger_localstorage/src/db/generic_controller.dart';
import 'package:geiger_localstorage/src/db/data/node.dart';
import 'package:geiger_localstorage/src/db/data/node_implementation.dart';
import 'package:geiger_localstorage/src/db/mapper/dummy_mapper.dart';
import 'package:test/test.dart';

class StorageListenerTest {}

void _setupTest(StorageController controller) {
  controller.zap();
}

class NodeListener with StorageListener {
  final Node _oldNode = NodeImpl('', 'testowner');
  final Node _newNode = NodeImpl('', 'testowner');

  Future<Node> get oldnode async {
    return await _oldNode.deepClone();
  }

  Future<Node> get newnode async {
    return await _newNode.deepClone();
  }

  @override
  Future<void> gotStorageChange(
      EventType event, Node? oldNode, Node? newNode) async {
    await _oldNode.update(oldNode ?? NodeImpl('', 'testowner'));
    await _newNode.update(newNode ?? NodeImpl('', 'testowner'));
  }
}

void _testRegisterDeRegisterListener(StorageController controller) {
  test('Register and de-register tests', () {
    _setupTest(controller);
    var sl = NodeListener();

    //create a dummy search criteria
    var sc = SearchCriteria();

    // register the listener
    controller.registerChangeListener(sl, sc);

    //deregister listener
    var scr = controller.deregisterChangeListener(sl);

    // check return values
    expect(scr.length, 1,
        reason:
            'returned array does not contain only one search criteria when de-registering an known listener');
    expect(scr.first, sc,
        reason:
            'returned array does not contain only one search criteria when de-registering a known listener');

    // check de-registering unregistered listener
    expect(() => controller.deregisterChangeListener(sl),
        throwsA(TypeMatcher<StorageException>()));
  });
}

void main() {
  var controller = GenericController('testOwner', DummyMapper('testowner'));

  _testRegisterDeRegisterListener(controller);
}
