// ignore_for_file: avoid_print

import 'dart:async';

import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:geiger_localstorage/src/db/data/timestamp.dart';
import 'package:test/test.dart';

void _callIncliningTests(bool requireInit) {
  test('test strict inclining timestamp with $requireInit', () {
    int lastTimestamp = 0;
    for (int i = 0; i < 1000; i++) {
      int ts = ExtendedTimestamp.now(requireInit);
      expect(ts > lastTimestamp, isTrue, reason: 'not strictly inclining');
    }
  });
  test('test follow of ExtendedTimestamp', () {
    String lastts = '';
    for (int i = 0; i < 1000; i++) {
      String ts = ExtendedTimestamp.convertToExtendedTimestamp(
          ExtendedTimestamp.now(requireInit));
      expect(ts.compareTo(lastts), 1, reason: 'not strictly inclining');
    }
  });
}

Future<void> main() async {
  group('un-inited timestamp tests', () {
    _callIncliningTests(false);
  });
  group('inited timestamp tests', () {
    StorageController c =
        GenericController('testowner', DummyMapper('testOwner'));
    test('testing un-inited timestamp exception', () async {
      await c.zap();
      ExtendedTimestamp.reset();
      expect(() => ExtendedTimestamp.now(),
          throwsA(const TypeMatcher<StorageException>()),
          reason: 'unexpectedly successful on un-inited ExtendedTimestamp');
    });
    test('Calling init', () async {
      await ExtendedTimestamp.initializeTimestamp(c);
    });
    _callIncliningTests(true);
  });
}
